import { injectable, inject } from "inversify";
import { CommandContribution, CommandRegistry, MenuContribution, MenuModelRegistry, MessageService } from "@theia/core/lib/common";
import { CommonMenus,FrontendApplication,FrontendApplicationContribution,PreferenceService } from "@theia/core/lib/browser";
import { MAIN_MENU_BAR } from '@theia/core';
import {FileResource } from '@theia/filesystem/lib/browser/file-resource';
import { WorkspaceService } from '@theia/workspace/lib/browser';
import { FileSystemWatcher } from '@theia/filesystem/lib/browser/filesystem-watcher';
import { FileSystem } from '@theia/filesystem/lib/common';
import URI from '@theia/core/lib/common/uri';
import { TerminalService } from '@theia/terminal/lib/browser/base/terminal-service';
import { TerminalWidget } from '@theia/terminal/lib/browser/base/terminal-widget';
import { TerminalWidgetFactoryOptions } from '@theia/terminal/lib/browser/terminal-widget-impl';
import { FrontendApplicationStateService } from '@theia/core/lib/browser/frontend-application-state';
import {MiniBrowserOpenHandler} from '@theia/mini-browser/lib/browser/mini-browser-open-handler';
import { OutlineViewContribution  } from '@theia/outline-view/lib/browser/outline-view-contribution';
import {ApplicationShell} from '@theia/core/lib/browser/shell/application-shell';
import { EditorCommandContribution } from '@theia/editor/lib/browser/editor-command';
import { QuickFileOpenService } from '@theia/file-search/lib/browser/quick-file-open';

export const Programmr_matplotmenuCommand = {
    id: 'Programmr_matplotmenu.command',
    label: "Shows a message"
};
 
export const RunExtensionCommand = {
    id: 'RunExtension.command',
    label: "View Graph"
};
export const CompileExtensionCommand = {
    id: 'CompileExtension.command',
    label: "Run"
};

export let configObj2:any = {};

export namespace RunMenus {
    export const RUN = [...MAIN_MENU_BAR, '1_run'];
    export const RUN_NEW = [...RUN, '3_run'];
    export const BUILD_NEW = [...RUN, '1_run'];	
    export const RUN_TASKS = [...RUN, '2_run'];
	    export const OPEN_NEW = [...RUN, '4_run'];
   
}

@injectable()
export class Programmr_matplotmenuCommandContribution implements CommandContribution, FrontendApplicationContribution, MenuContribution {
        private configObj: any = '';
	 interval:any;
	  @inject(FrontendApplicationStateService)
    protected readonly stateService: FrontendApplicationStateService;
 
 @inject(WorkspaceService)
    protected readonly workspaceService: WorkspaceService;		
  @inject(PreferenceService)
    protected readonly preferencesService: PreferenceService;	
 @inject(QuickFileOpenService)
 
    protected readonly quickFileOpenService: QuickFileOpenService;	
    constructor(
        @inject(MessageService) private readonly messageService: MessageService,
	    @inject(TerminalService) protected readonly terminalService: TerminalService,
		@inject(FileSystem) protected readonly fileSystem: FileSystem,
		@inject(FileSystemWatcher) protected readonly watcher: FileSystemWatcher,
		@inject(MiniBrowserOpenHandler) protected readonly miniBrowser: MiniBrowserOpenHandler,
	    @inject(ApplicationShell) protected readonly shell: ApplicationShell,	
		@inject(OutlineViewContribution) protected readonly outlineViewContribution: OutlineViewContribution,	
		@inject(MenuModelRegistry) protected readonly menuRegistry: MenuModelRegistry,		
    ) { }
    
	
	 protected setConfigObject(obj){
	 this.configObj = obj;
	}
	
	 protected getConfigObject(): Object{
	 return this.configObj;
	}
	
		    async onStart(app: FrontendApplication): Promise<void> {         
		   this.shell.expandPanel('left');	
            this.stateService.reachedState('ready').then(
                a => {
				    	console.log("strt:2");
						//set autosave on
						 //this.shell.expandPanel('left');
						// this.shell.expandPanel('right');
						// this.outlineViewContribution.openView(); 
						this.preferencesService.set(EditorCommandContribution.AUTOSAVE_PREFERENCE,'on');						
						const current = this.workspaceService.workspace;
                       //get the current workspace project path and open the default file

					   	try{
								//console.log("menu"+document.querySelectorAll(".p-MenuBar-item"));
								let node : NodeListOf<Element> = document.querySelectorAll(".p-MenuBar-item");
								let el = node[node.length - 1] as HTMLElement;
								 el.style.display="none";
					  	}catch(e){
							console.log(e);
						} 
					  // try{
					   //this.quickFileOpenService.openFile(new URI(current.uri+"/src/app/app.component.ts"));
					  // }catch(e){
					   //  console.log("open err"+e); 
					   //}
					   this.openDefaultFile(current.uri);
					  	 
				}
            );
       
    }
	
			async openDefaultFile(path: string): Promise<void>{
			try{
			const uri = new URI(path+"/.pmainfile");
				 const resource = new FileResource(uri, this.fileSystem, this.watcher);
				 const current = this.workspaceService.workspace;
				 const projectPath = current.uri.toString().replace('file://','');
				let contents = await resource.readContents();				
				contents = contents.replace(/\n/g,'');
				contents = contents.replace(/\r/g,'');
				this.configObj = JSON.parse(contents);
				configObj2 = JSON.parse(contents);;
				this.setConfigObject(this.configObj);
				 if("mainfile" in this.configObj){
					this.quickFileOpenService.openFile(new URI(path+"/"+this.configObj.mainfile));
				 }
				 if("previewApp" in this.configObj){
					console.log("preview:true");
					//preivew already deployed android app in appetize
					 if("runmenucommand" in this.configObj){
						this.runTerminalCommand(this.configObj.runmenucommand+" "+projectPath);
				 }
				
					this.monitorOutputUrl();
				 }
				 
				  if("outputtype" in this.configObj && this.configObj.outputtype == "graph"){
				      this.menuRegistry.registerMenuAction(RunMenus.RUN_NEW, {
							commandId: RunExtensionCommand.id,
							label: 'View Graph',
							order: '1'
						});	
				 }
				 
			}catch(e){
			 //console.log(e);
			 console.log("exceptio .. no config open default index.js file")	 			 
			}	 
	}
	
	async getConfiguredCommandFor(command: string): Promise<void>{
			try{
			console.log("configObj2="+configObj2);
			const current = this.workspaceService.workspace;
			const projectPath = current.uri.toString().replace('file://','');			
			const uri = new URI(current.uri+"/.pmainfile");
				 const resource = new FileResource(uri, this.fileSystem, this.watcher);
				let contents = await resource.readContents();
				contents = contents.replace(/\n/g,'');
				contents = contents.replace(/\r/g,'');
				this.configObj = JSON.parse(contents);
		
			    if(command == 'run' ){ 
				 if("runmenucommand" in this.configObj){
					this.runTerminalCommand(this.configObj.runmenucommand+" "+projectPath);
					if("outputtype" in this.configObj && this.configObj.outputtype == "graph")
						this.monitorOutputUrl();
				 }
                 }
				 else 
				 if(command == 'compile' ){ 
						if("compilemenucommand" in this.configObj){
							this.runTerminalCommand(this.configObj.compilemenucommand);
						}
                 } 
				 
				 
			}catch(e){
			 console.log(e);
			}	 
	}
	
	async getOutputUrl(command: string): Promise<void>{
			try{
			console.log("configObj2="+configObj2);
			const current = this.workspaceService.workspace;
			const uri = new URI(current.uri+"/.outputurl");
				 const resource = new FileResource(uri, this.fileSystem, this.watcher);
				let contents = await resource.readContents();
				contents = contents.replace(/\n/g,'');
				contents = contents.replace(/\r/g,'');
				//let outputObj = JSON.parse(contents);
				 if(contents.length>1){
					 clearInterval(this.interval);
					console.log("publicKey found"+window.location.href);
					let windowUrl = window.location.href;
					windowUrl = windowUrl.replace("/#/","/mini-browser/");
					if("graphfile" in this.configObj){					
						windowUrl+="/"+this.configObj.graphfile;
					}
					else
					windowUrl+="/plot.png";
					
					this.openUrlInPreview(windowUrl);
				 }
          
				 else { 
							console.log("entry not found");						
                 }
				 
				 
			}catch(e){
			 console.log(e);
			}	 
	}
	
    registerCommands(registry: CommandRegistry): void {
        registry.registerCommand(Programmr_matplotmenuCommand, {
            execute: () => this.messageService.info('Hello World!')
        });
		  registry.registerCommand(CompileExtensionCommand, {
            execute: () => {
					//this.messageService.info('Running...');
					this.getConfiguredCommandFor('run');
					//this.openUrlInPreview('https://appetize.io/embed/gzghhwdtg34e8mbv5w4vfyfu7w?device=nexus5&scale=75&orientation=portrait&osVersion=8.1');
					
			}
        });		 
        registry.registerCommand(RunExtensionCommand, {
            execute: () => {
					//this.messageService.info('Loading...');
					//this.getConfiguredCommandFor('run');
					//this.openUrlInPreview('https://appetize.io/embed/gzghhwdtg34e8mbv5w4vfyfu7w?device=nexus5&scale=75&orientation=portrait&osVersion=8.1');
					//this.monitorOutputUrl();
						let windowUrl = window.location.href;
					windowUrl = windowUrl.replace("/#/","/mini-browser/");
					if("graphfile" in this.configObj){					
						windowUrl+="/"+this.configObj.graphfile;
					}
					else
					windowUrl+="/plot.png";
					
					this.openUrlInPreview(windowUrl);
					
			}
        });
    }
	
	async runTerminalCommand(command: string): Promise<void> {
					    let terminal = this.terminalService.currentTerminal;
				if (!terminal) {
					terminal = <TerminalWidget>await this.terminalService.newTerminal(<TerminalWidgetFactoryOptions>{ created: new Date().toString() });
					await terminal.start();
					this.terminalService.activateTerminal(terminal);
				}
				terminal.sendText(command+"\n");  
		 
	 }
	
	//dj:
		monitorOutputUrl(){
			this.interval = setInterval(()=>{
				this.getOutputUrl('run');
			},1000);
		
		}
		async openUrlInPreview(url): Promise<void> {
		    if("viewtype" in configObj2 && configObj2.viewtype == "browser"){
			await this.miniBrowser.openPreview(url);
			}
			else
			if("viewtype" in configObj2 && configObj2.viewtype == "file"){
					const current = this.workspaceService.workspace;
					 this.shell.collapsePanel('bottom');
					this.quickFileOpenService.openFile(new URI(current.uri+"/"+configObj2.graphfile));
				 }
		}
 

    	 registerMenus(menus: MenuModelRegistry): void {
	menus.registerSubmenu(RunMenus.RUN, 'Run');
		menus.registerMenuAction(RunMenus.BUILD_NEW, {
            commandId: CompileExtensionCommand.id,
            label: 'Run',
            order: '0' 
        });	
					
     
		
		
        menus.registerMenuAction(CommonMenus.EDIT_FIND, {
            commandId: RunExtensionCommand.id,
            label: 'View Graph'
        });
        menus.registerMenuAction(CommonMenus.EDIT_FIND, {
            commandId: CompileExtensionCommand.id,
            label: 'Run'
        });	
	}
}